@@ -9,13 +9,18 @@ import sys
 import logging
 import argparse
 
+import pefile
+
 from virt.firmware.efi import ucs16
+from virt.firmware.efi import guids
 from virt.firmware.efi import devpath
 from virt.firmware.efi import bootentry
 
 from virt.firmware.bootcfg import bootcfg
 from virt.firmware.bootcfg import linuxcfg
 
+from virt.peutils import pesign
+
 
 ########################################################################
 # main
@@ -32,28 +37,67 @@ def update_next_or_order(cfg, options, nr):
             cfg.linux_update_order()
 
 
-def add_uki(cfg, options):
-    if not options.shim:
-        logging.error('shim binary not specified')
-        sys.exit(1)
-    if not options.title:
-        logging.error('entry title not specified')
-        sys.exit(1)
+def firmware_loads_efi_binary(cfg, efibinary):
+    if not cfg.secureboot:
+        return True
+    if pesign.cryptography_major < 40:
+        # can't use pesign.pe_check_cert() -> play safe
+        return False
+    db = cfg.varstore.get_variable('db', guids.EfiImageSecurityDatabase)
+    pe = pefile.PE(efibinary)
+    siglist = pesign.pe_type2_signatures(pe)
+    if pesign.pe_check_cert(siglist, db):
+        return True
+    return False
 
-    efiuki = linuxcfg.LinuxEfiFile(options.adduki)
-    nr = cfg.find_uki_entry(efiuki.efi_filename())
-    if nr is not None:
-        logging.info('Entry exists (Boot%04X)', nr)
-    else:
+
+def create_boot_entry(efiuki, options):
+    if options.shim:
         efishim = linuxcfg.LinuxEfiFile(options.shim)
         if efishim.device != efiuki.device:
             logging.error('shim and uki are on different filesystems')
             sys.exit(1)
-        optdata = ucs16.from_string(efiuki.efi_filename())
+        if options.cmdline:
+            optdata = ucs16.from_string(efiuki.efi_filename() + ' ' + options.cmdline)
+        else:
+            optdata = ucs16.from_string(efiuki.efi_filename())
         entry = bootentry.BootEntry(title = ucs16.from_string(options.title),
                                     attr = bootentry.LOAD_OPTION_ACTIVE,
                                     devicepath = efishim.dev_path_file(),
                                     optdata = bytes(optdata))
+    else:
+        if options.cmdline:
+            optdata = ucs16.from_string(options.cmdline)
+            entry = bootentry.BootEntry(title = ucs16.from_string(options.title),
+                                        attr = bootentry.LOAD_OPTION_ACTIVE,
+                                        devicepath = efiuki.dev_path_file(),
+                                        optdata = bytes(optdata))
+        else:
+            entry = bootentry.BootEntry(title = ucs16.from_string(options.title),
+                                        attr = bootentry.LOAD_OPTION_ACTIVE,
+                                        devicepath = efiuki.dev_path_file())
+    return entry
+
+
+def add_uki(cfg, options):
+    if not options.shim and not firmware_loads_efi_binary(cfg, options.adduki):
+        logging.error('shim binary needed but not found or specified')
+        sys.exit(1)
+    if not options.title:
+        logging.error('entry title not specified')
+        sys.exit(1)
+    if options.cmdline and cfg.secureboot:
+        logging.warning('Overriding built-in UKI cmdline is not possible'
+                        ' when Secure Boot is enabled')
+
+    efiuki = linuxcfg.LinuxEfiFile(options.adduki)
+    nr = cfg.find_uki_entry(efiuki.efi_filename())
+    if nr is None:
+        nr = cfg.find_devpath_entry(efiuki.dev_path_file())
+    if nr is not None:
+        logging.info('Entry exists (Boot%04X)', nr)
+    else:
+        entry = create_boot_entry(efiuki, options)
         logging.info('Create new entry: %s', str(entry))
         nr = cfg.add_entry(entry)
         logging.info('Added entry (Boot%04X)', nr)
@@ -66,6 +110,8 @@ def add_uki(cfg, options):
 def update_uki(cfg, options):
     efiuki = linuxcfg.LinuxEfiFile(options.updateuki)
     nr = cfg.find_uki_entry(efiuki.efi_filename())
+    if nr is None:
+        nr = cfg.find_devpath_entry(efiuki.dev_path_file())
     if nr is None:
         logging.error('No entry found for %s', options.updateuki)
         sys.exit(1)
@@ -76,6 +122,8 @@ def update_uki(cfg, options):
 def remove_uki(cfg, options):
     efiuki = linuxcfg.LinuxEfiFile(options.removeuki)
     nr = cfg.find_uki_entry(efiuki.efi_filename())
+    if nr is None:
+        nr = cfg.find_devpath_entry(efiuki.dev_path_file())
     if nr is None:
         logging.warning('No entry found for %s', options.removeuki)
         return
@@ -89,8 +137,8 @@ def remove_uki(cfg, options):
 
 
 def boot_success(cfg, options):
-    if cfg.bcurr == cfg.blist[0]:
-        logging.info('No update needed, BootCurrent already comes first in BootOrder.')
+    if cfg.bcurr in cfg.blist:
+        logging.info('No update needed, BootCurrent is already in BootOrder.')
         return
     logging.info('Add BootCurrent (Boot%04X) to BootOrder', cfg.bcurr)
     cfg.set_boot_order(cfg.bcurr, 0)
@@ -185,10 +233,12 @@ def main():
                        help = 'update boot entry for UKI image FILE', metavar = 'FILE')
     group.add_argument('--remove-uki', dest = 'removeuki', type = str,
                        help = 'remove boot entry for UKI image FILE', metavar = 'FILE')
+    group.add_argument('--cmdline', dest = 'cmdline', type = str,
+                       help = 'override UKIs cmdline when adding boot entry '
+                       '(ignored when Secure Boot is enabled)', metavar = 'CMDLINE')
     group.add_argument('--boot-ok', '--boot-successful', dest = 'bootok',
                        action = 'store_true', default = False,
-                       help = 'boot is successful, update BootOrder to have '
-                       'current entry listed first.')
+                       help = 'boot is successful, add BootCurrent to BootOrder.')
     group.add_argument('--update-csv', dest = 'updatecsv',
                        action = 'store_true', default = False,
                        help = 'update BOOT.CSV')
@@ -251,7 +301,7 @@ def main():
         remove_uki(cfg, options)
     elif options.bootok:
         boot_success(cfg, options)
-        if options.updatecsv:
+        if options.updatecsv and options.shim:
             update_boot_csv(cfg, options)
     elif options.updatecsv:
         update_boot_csv(cfg, options)
